From 1804702b5e3c85c1a16014d62365a29d0a6d0c75 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Micha=C5=82=20G=C3=B3rny?= <mgorny@gentoo.org>
Date: Thu, 30 Jan 2020 06:15:27 +0100
Subject: [PATCH] Skip tests failing on PyPy

--- a/doc/element_classes.txt
+++ b/doc/element_classes.txt
@@ -576,10 +576,10 @@ subclasses for elements of this namespace:
 
   >>> print(honk_element[0].honk())
   HONK
-  >>> print(honk_element[0].honking)
+  >>> print(honk_element[0].honking)  # doctest: +ELLIPSIS
   Traceback (most recent call last):
   ...
-  AttributeError: 'HonkNSElement' object has no attribute 'honking'
+  AttributeError: 'HonkNSElement' object has no attribute 'honking'...
 
   >>> print(honk_element[1].text)  # uses fallback for non-elements
   comment
--- a/src/lxml/tests/test_elementtree.py
+++ b/src/lxml/tests/test_elementtree.py
@@ -26,7 +26,7 @@ from .common_imports import (
     BytesIO, etree, HelperTestCase,
     ElementTree, cElementTree, ET_VERSION, CET_VERSION,
     filter_by_version, fileInTestDir, canonicalize, tmpfile,
-    _str, _bytes, unicode, IS_PYTHON2
+    _str, _bytes, unicode, IS_PYTHON2, IS_PYPY
 )
 
 if cElementTree is not None and (CET_VERSION <= (1,0,7) or sys.version_info[0] >= 3):
@@ -2734,6 +2734,7 @@ class _ETreeTestCaseBase(HelperTestCase):
         self.assertEqual('TEST', root2[0].get('{%s}a' % ns_href))
 
     required_versions_ET['test_register_namespace'] = (1,3)
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_register_namespace(self):
         # ET 1.3+
         Element = self.etree.Element
--- a/src/lxml/tests/test_errors.py
+++ b/src/lxml/tests/test_errors.py
@@ -11,7 +11,7 @@ import unittest
 import sys, gc, os.path
 from lxml import etree
 
-from .common_imports import HelperTestCase
+from .common_imports import HelperTestCase, IS_PYPY
 
 
 class ErrorTestCase(HelperTestCase):
@@ -25,6 +25,7 @@ class ErrorTestCase(HelperTestCase):
     def test_empty_parse(self):
         self.assertRaises(etree.XMLSyntaxError, etree.fromstring, '')
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_element_cyclic_gc_none(self):
         # test if cyclic reference can crash etree
         Element = self.etree.Element
--- a/src/lxml/tests/test_http_io.py
+++ b/src/lxml/tests/test_http_io.py
@@ -11,10 +11,11 @@ import textwrap
 import sys
 import gzip
 
-from .common_imports import etree, HelperTestCase, BytesIO, _bytes
+from .common_imports import etree, HelperTestCase, BytesIO, _bytes, IS_PYPY
 from .dummy_http_server import webserver, HTTPRequestCollector
 
 
+@unittest.skipIf(IS_PYPY, "broken on pypy")
 class HttpIOTestCase(HelperTestCase):
     etree = etree
 
--- a/src/lxml/tests/test_nsclasses.py
+++ b/src/lxml/tests/test_nsclasses.py
@@ -9,7 +9,7 @@ from __future__ import absolute_import
 
 import unittest
 
-from .common_imports import etree, HelperTestCase, _bytes, make_doctest
+from .common_imports import etree, HelperTestCase, _bytes, make_doctest, IS_PYPY
 
 class ETreeNamespaceClassesTestCase(HelperTestCase):
     
@@ -46,6 +46,7 @@ class ETreeNamespaceClassesTestCase(HelperTestCase):
         self.Namespace('ns02').clear()
         self.Namespace('ns03').clear()
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_ns_classes(self):
         bluff_dict = {'bluff' : self.bluff_class}
         maeh_dict  = {'maeh'  : self.maeh_class}
--- a/src/lxml/tests/test_objectify.py
+++ b/src/lxml/tests/test_objectify.py
@@ -11,7 +11,8 @@ import random
 import unittest
 
 from .common_imports import (
-    etree, HelperTestCase, fileInTestDir, doctest, make_doctest, _bytes, _str, BytesIO
+    etree, HelperTestCase, fileInTestDir, doctest, make_doctest, _bytes, _str, BytesIO,
+    IS_PYPY
 )
 
 from lxml import objectify
@@ -215,11 +216,13 @@ class ObjectifyTestCase(HelperTestCase):
         expected.update(DEFAULT_NSMAP)
         self.assertEqual(root.value.nsmap, expected)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_date_element_efactory_text(self):
         # ObjectifiedDataElement can also be used as E-Factory
         value = objectify.ObjectifiedDataElement('test', 'toast')
         self.assertEqual(value.text, 'testtoast')
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_date_element_efactory_tail(self):
         # ObjectifiedDataElement can also be used as E-Factory
         value = objectify.ObjectifiedElement(objectify.ObjectifiedDataElement(), 'test', 'toast')
@@ -376,6 +379,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual("4", getattr(root.c1, "{}c2").text)
         self.assertEqual("0", getattr(root.c1, "c2").text)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_setattr(self):
         for val in [
             2, 2**32, 1.2, "Won't get fooled again", 
@@ -811,6 +815,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(3, len(root.findall(".//b")))
         self.assertEqual(2, len(root.findall("b")))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_build_tree(self):
         root = self.Element('root')
         root.a = 5
@@ -840,6 +845,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(value, None)
         self.assertEqual(value.get(XML_SCHEMA_NIL_ATTR), "true")
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_bool(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -877,6 +883,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(objectify.DataElement(False).text, "false")
         self.assertEqual(objectify.DataElement(True).text, "true")
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -884,6 +891,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = "test"
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_intliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -891,6 +899,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = "3"
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_floatliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -898,6 +907,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = "3.72"
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_mul(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -910,6 +920,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertRaises(TypeError, operator.mul, root.s, "honk")
         self.assertRaises(TypeError, operator.mul, "honk", root.s)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_add(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -920,6 +931,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual("test" + s, root.s + s)
         self.assertEqual(s + "test", s + root.s)
             
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_mod(self):
         s = "%d %f %s %r"
         el = objectify.DataElement(s)
@@ -985,6 +997,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertTrue(isinstance(value, objectify.StringElement))
         self.assertEqual(value, "3.20")
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_ustr(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -992,6 +1005,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = _str("test")
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_ustr_intliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -999,6 +1013,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = _str("3")
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_ustr_floatliteral(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -1006,6 +1021,7 @@ class ObjectifyTestCase(HelperTestCase):
         root.s = _str("3.72")
         self.assertTrue(isinstance(root.s, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_ustr_mul(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -1018,6 +1034,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertRaises(TypeError, operator.mul, root.s, _str("honk"))
         self.assertRaises(TypeError, operator.mul, _str("honk"), root.s)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_ustr_add(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -1043,6 +1060,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertTrue(isinstance(value, objectify.StringElement))
         self.assertEqual(value, _str("3.20"))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_int(self):
         Element = self.Element
         root = Element("{objectified}root")
@@ -1059,6 +1077,7 @@ class ObjectifyTestCase(HelperTestCase):
         value = objectify.DataElement(123)
         self.assertEqual(hash(value), hash(123))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_float(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -1075,6 +1094,7 @@ class ObjectifyTestCase(HelperTestCase):
         value = objectify.DataElement(5.5)
         self.assertEqual(hash(value), hash(5.5))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_float_precision(self):
         # test not losing precision by shortened float str() value
         # repr(2.305064300557): '2.305064300557'
@@ -1094,6 +1114,7 @@ class ObjectifyTestCase(HelperTestCase):
         s = "2.305064300557"
         self.assertEqual(objectify.FloatElement(s), float(s))
   
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_float_precision_consistency(self):
         # test consistent FloatElement values for the different instantiation
         # possibilities
@@ -1180,6 +1201,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(value.text, None)
         self.assertEqual(value.pyval, None)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_unregistered(self):
         Element = self.Element
         SubElement = self.etree.SubElement
@@ -1342,6 +1364,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(["why", "try"],
                           strs)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_str_cmp(self):
         XML = self.XML
         root = XML(_bytes('<root><b>test</b><b>taste</b><b></b><b/></root>'))
@@ -1369,6 +1392,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertEqual(root.b, "")
         self.assertEqual("", root.b)
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_int_cmp(self):
         XML = self.XML
         root = XML(_bytes('<root><b>5</b><b>6</b></root>'))
@@ -1391,6 +1415,7 @@ class ObjectifyTestCase(HelperTestCase):
         
     # float + long share the NumberElement implementation with int
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_type_bool_cmp(self):
         XML = self.XML
         root = XML(_bytes('<root><b>false</b><b>true</b></root>'))
@@ -2060,6 +2085,7 @@ class ObjectifyTestCase(HelperTestCase):
                           before = [objectify.getRegisteredTypes()[0].name],
                           after  = [objectify.getRegisteredTypes()[1].name])
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_registered_type_stringify(self):
         from datetime import datetime
         def parse_date(value):
@@ -2530,46 +2556,55 @@ class ObjectifyTestCase(HelperTestCase):
 
     # E-Factory tests, need to use sub-elements as root element is always
     # type-looked-up as ObjectifiedElement (no annotations)
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_int(self):
         E = objectify.E
         root = E.root(E.val(23))
         self.assertTrue(isinstance(root.val, objectify.IntElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_float(self):
         E = objectify.E
         root = E.root(E.val(233.23))
         self.assertTrue(isinstance(root.val, objectify.FloatElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_str(self):
         E = objectify.E
         root = E.root(E.val("what?"))
         self.assertTrue(isinstance(root.val, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_unicode(self):
         E = objectify.E
         root = E.root(E.val(_str("blöödy häll", encoding="ISO-8859-1")))
         self.assertTrue(isinstance(root.val, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_bool(self):
         E = objectify.E
         root = E.root(E.val(True))
         self.assertTrue(isinstance(root.val, objectify.BoolElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_none(self):
         E = objectify.E
         root = E.root(E.val(None))
         self.assertTrue(isinstance(root.val, objectify.NoneElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_value_concatenation(self):
         E = objectify.E
         root = E.root(E.val(1, "foo", 2.0, "bar ", True, None))
         self.assertTrue(isinstance(root.val, objectify.StringElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_attrib(self):
         E = objectify.E
         root = E.root(foo="bar")
         self.assertEqual(root.get("foo"), "bar")
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_nested(self):
         E = objectify.E
         DataElement = objectify.DataElement
@@ -2584,6 +2619,7 @@ class ObjectifyTestCase(HelperTestCase):
         self.assertTrue(isinstance(root.value[0], objectify.IntElement))
         self.assertTrue(isinstance(root.value[1], objectify.FloatElement))
 
+    @unittest.skipIf(IS_PYPY, "broken on pypy")
     def test_efactory_subtype(self):
         class Attribute(objectify.ObjectifiedDataElement):
             def __init__(self):
@@ -2739,7 +2775,8 @@ def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ObjectifyTestCase)])
     suite.addTests(doctest.DocTestSuite(objectify))
-    suite.addTests([make_doctest('../../../doc/objectify.txt')])
+    if not IS_PYPY:
+        suite.addTests([make_doctest('../../../doc/objectify.txt')])
     return suite
 
 if __name__ == '__main__':
--- a/src/lxml/tests/test_xpathevaluator.py
+++ b/src/lxml/tests/test_xpathevaluator.py
@@ -8,7 +8,7 @@ from __future__ import absolute_import
 
 import unittest, sys
 
-from .common_imports import etree, HelperTestCase, _bytes, BytesIO, doctest, make_doctest
+from .common_imports import etree, HelperTestCase, _bytes, BytesIO, doctest, make_doctest, IS_PYPY
 
 
 class ETreeXPathTestCase(HelperTestCase):
@@ -740,8 +740,9 @@ def test_suite():
         suite.addTests([unittest.makeSuite(ETreeXPathExsltTestCase)])
     suite.addTests([unittest.makeSuite(ETreeETXPathClassTestCase)])
     suite.addTests([doctest.DocTestSuite()])
-    suite.addTests(
-        [make_doctest('../../../doc/xpathxslt.txt')])
+    if not IS_PYPY:
+        suite.addTests(
+            [make_doctest('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
--- a/src/lxml/tests/test_xslt.py
+++ b/src/lxml/tests/test_xslt.py
@@ -17,6 +17,8 @@ from textwrap import dedent
 from tempfile import NamedTemporaryFile, mkdtemp
 
 is_python3 = sys.version_info[0] >= 3
+is_pypy = (getattr(sys, 'implementation', None) == 'pypy' or
+           getattr(sys, 'pypy_version_info', None) is not None)
 
 try:
     unicode
@@ -2098,8 +2100,9 @@ def test_suite():
         suite.addTests([unittest.makeSuite(Py3XSLTTestCase)])
     suite.addTests(
         [make_doctest('../../../doc/extensions.txt')])
-    suite.addTests(
-        [make_doctest('../../../doc/xpathxslt.txt')])
+    if not is_pypy:
+        suite.addTests(
+            [make_doctest('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
-- 
2.41.0
